@@ -8,6 +8,22 @@ logging.basicConfig(level=logging.DEBUG)
 
 class BrowserIntegrationTest(TestCase):
 
+    def test_simple_example(self):
+        # Create a new instance of the Firefox driver
+        driver = webdriver.Firefox()
+
+        # Go to the Google home page
+        driver.get('https://www.google.com')
+
+        # Access requests via the `requests` attribute
+        for request in driver.requests:
+            if request.response:
+                print(
+                    request.url,
+                    request.response.status_code,
+                    request.response.headers['Content-Type']
+                )
+
     def test_firefox_can_access_requests(self):
         url = 'https://www.python.org/'
         driver = webdriver.Firefox()
@@ -24,7 +40,6 @@ class BrowserIntegrationTest(TestCase):
         url = 'https://www.python.org/'
         options = {
             'backend': 'mitmproxy',
-            'mitmproxy_log_level': 'INFO',
         }
         driver = webdriver.Firefox(seleniumwire_options=options)
         driver.get(url)
@@ -38,10 +53,7 @@ class BrowserIntegrationTest(TestCase):
 
     def test_chrome_can_access_requests(self):
         url = 'https://www.wikipedia.org/'
-        sw_options = {
-            'backend': 'mitmproxy'
-        }
-        driver = webdriver.Chrome(seleniumwire_options=sw_options)
+        driver = webdriver.Chrome()
         driver.get(url)
 
         request = driver.wait_for_request(url)
@@ -81,6 +93,44 @@ class BrowserIntegrationTest(TestCase):
 
         driver.quit()
 
+    def test_intercept_request(self):
+        url = 'https://www.python.org'
+        user_agent = 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) ' \
+                     'Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100'
+        driver = webdriver.Firefox()
+
+        def intercept(req):
+            del req.headers['User-Agent']
+            req.headers['User-Agent'] = user_agent
+            req.params = {**req.params, 'foo': 'bar'}
+
+        driver.request_interceptor = intercept
+
+        driver.get(url)
+
+        request = driver.wait_for_request(url)
+
+        self.assertEqual(user_agent, request.headers['User-Agent'])
+
+        driver.quit()
+
+    def test_intercept_response(self):
+        url = 'https://www.guardian.co.uk'
+        driver = webdriver.Firefox()
+
+        def intercept(req, res):
+            res.headers['X-Foo'] = 'bar'
+
+        driver.response_interceptor = intercept
+
+        driver.get(url)
+
+        request = driver.wait_for_request(url)
+
+        self.assertEqual('bar', request.response.headers['X-Foo'])
+
+        driver.quit()
+
     def test_modify_user_agent(self):
         user_agent = 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) ' \
                      'Chrome/28.0.1500.52 Safari/537.36 OPR/15.0.1147.100'
@@ -118,7 +168,7 @@ class BrowserIntegrationTest(TestCase):
         }
         driver.get('https://httpbin.org/get?foo=bar')
 
-        request = driver.wait_for_request('https://httpbin.org/get?foo=baz')
+        request = driver.wait_for_request(r'https://httpbin.org/get\?foo=baz')
 
         self.assertEqual({'foo': 'baz'}, request.params)
 
@@ -127,7 +177,6 @@ class BrowserIntegrationTest(TestCase):
     def test_modify_querystring(self):
         options = {
             'backend': 'mitmproxy',
-            'mitmproxy_log_level': 'INFO',
             'disable_encoding': True,
             'proxy': {
                 'https': 'https://localhost:8080'
@@ -137,7 +186,7 @@ class BrowserIntegrationTest(TestCase):
         driver.querystring_overrides = 'foo=baz'
         driver.get('https://httpbin.org/get?foo=bar')
 
-        request = driver.wait_for_request('https://httpbin.org/get?foo=baz')
+        request = driver.wait_for_request(r'https://httpbin.org/get\?foo=baz')
 
         self.assertEqual({'foo': 'baz'}, request.params)
 
@@ -154,3 +203,18 @@ class BrowserIntegrationTest(TestCase):
         driver.wait_for_request('https://www.wikipedia.org/')  # Should find www.wikipedia.org
 
         driver.quit()
+
+    def test_custom_response_handler(self):
+        """NOTE: this is being deprecated. Use driver.response_interceptor."""
+        def custom(req, req_body, res, res_body):
+            print(f'res_body length: {len(res_body)}')
+
+        options = {
+            'custom_response_handler': custom
+        }
+
+        driver = webdriver.Firefox(seleniumwire_options=options)
+
+        driver.get('https://www.python.org/')
+
+        driver.quit()
